<?php
if(file_exists(__DIR__ . \DIRECTORY_SEPARATOR . 'PaykeeperPayment.class.php'))
require_once __DIR__ . \DIRECTORY_SEPARATOR . 'PaykeeperPayment.class.php';
/*
Plugins "PayKeeper" for Shop-Script

 * @author PayKeeper
 * @name PayKeeper
 * @description Плагин оплаты через PayKepeer.
 *
 * Поля, доступные в виде параметров настроек плагина, указаны в файле lib/config/settings.php.
 * @property-read string $pk_url_link
 * @property-read string $pk_secret_key
*/
class paykeeper_methodPayment extends waPayment implements waIPayment 
{

    private $fiscal_cart = array(); //fz54 cart
    private $order_total = 0; //order total sum
    private $shipping_price = 0; //shipping price
    private $use_taxes = false;
    private $use_delivery = false;
    private $order_params = NULL;
    private $discounts = array();
    private $submit_button_class = "";
    protected $table = 'shop_order';
    //currency (only RUB available)
    public function allowedCurrency()
    {
        return $currency = 'RUB';
    }


    //redirect to payment form page
    public function payment($payment_form_data, $order_data, $form_html = true)
    {
	    $pk_obj = new PaykeeperPayment; 
        //required
        if (empty($order_data['description'])) {
            $order_data['description'] = 'Заказ '.$order_data['order_id'];
        }

        //initialize order object
        $order = waOrder::factory($order_data);

        $customer = new waContact($order_data["contact_id"]);

        //set order parameters
        $orderid = $order_data["id_str"] . "|" .
                   $order_data["order_id"] . "|" .
                   $this->app_id . "|" .
                   $this->merchant_id;

        $email_data = $customer->get("email");
        $email = $email_data[0]["value"];
        $phone_data = $customer->get("phone");
        $phone = $phone_data[0]["value"];

        $pk_obj->setOrderParams($order->total,                        //sum
                              $customer->get("name"),               //clientid
                              $orderid,                             //orderid
                              $email,                               //client_email
                              $phone,                               //client_phone
                              "",                                   //service_name
                              $this->pk_url_link,                   //payment form url
                              $this->pk_secret_key                  //secret key
        );

        //GENERATE FZ54 CART
        $cart_data = $order_data["items"];
        $item_index = 0;
        
        foreach ($cart_data as $cart_item) {
            $name = $cart_item["name"];
            $quantity = $cart_item["quantity"];
            if($cart_item["tax_included"] == 0){
                $price = $cart_item["price"] + $cart_item["price"]/100*$cart_item["tax_rate"]; 
            }
            else {
                $price = $cart_item["price"];

            }
            if ($quantity == 1 && $pk_obj->single_item_index < 0)
                $pk_obj->single_item_index = $item_index;
            if ($quantity > 1 && $pk_obj->more_then_one_item_index < 0)
                $pk_obj->more_then_one_item_index = $item_index;
            $sum = number_format($price*$quantity, 2, ".", "");
            $tax_rate = $cart_item["tax_rate"];
            $taxes = $pk_obj->setTaxes($tax_rate);
            $pk_obj->updateFiscalCart($pk_obj->getPaymentFormType(), 
                                    $name, $price, $quantity, $sum, $taxes["tax"]);
                                   
            $item_index++;
        }
            
        //add shipping parameters to cart
        $shipping_tax_rate = 0;
        $shipping_taxes = array("tax" => "none", "tax_sum" => 0);
        $shipping_tax_rate = $order["shipping_tax_rate"];
        if ($order["shipping_tax_included"] == 0){
            $shipping_tax_final = $order["shipping"] + ($shipping_tax_rate * $order["shipping"])/100;
        }
        else {
            $shipping_tax_final = $order["shipping"];
        }
        $pk_obj->setShippingPrice($shipping_tax_final);
        $shipping_name = $order["shipping_name"];
        $shipping_taxes = $pk_obj->setTaxes($shipping_tax_rate);
               
        if ($pk_obj->getShippingPrice() > 0) {
           
            $pk_obj->setUseDelivery(); //for precision correct check
            $pk_obj->updateFiscalCart($pk_obj->getPaymentFormType(),
                                    $shipping_name,
                                    $pk_obj->getShippingPrice(), 1, 
                                    $pk_obj->getShippingPrice(), 
                                    $shipping_taxes["tax"]);
        
             
            
            }
        $pk_obj->setOrderTotal($order["total"] + ($order["total"] - $order["subtotal"] - $order["shipping"]));                                       
        
	//set discounts
        $pk_obj->setDiscounts(array_key_exists("coupon_discount", $order["params"]) || $this->pk_force_discounts_check ==1);
        
        //handle possible precision problem
        $pk_obj->correctPrecision();

        $fiscal_cart_encoded = json_encode($pk_obj->getFiscalCart());

        //generate payment form
        $form = "";

        if ($pk_obj->getPaymentFormType() == "create") { //create form
            $to_hash = number_format($pk_obj->getOrderTotal(), 2, ".", "")                .
                        $pk_obj->getOrderParams("clientid")     .
                        $pk_obj->getOrderParams("orderid")      .
                        $pk_obj->getOrderParams("service_name") .
                        $pk_obj->getOrderParams("client_email") .
                        $pk_obj->getOrderParams("client_phone") .
                        $pk_obj->getOrderParams("secret_key");
            $sign = hash ('sha256' , $to_hash);
            
            $this->setSubmitButtonCSSClass("large bold");
            $form = '
                <h3>Сейчас Вы будете перенаправлены на страницу банка.</h3> 
                <form name="payment" id="pay_form" action="'.$pk_obj->getOrderParams("form_url").'" accept-charset="utf-8" method="post">
                <input type="hidden" name="sum" value = "'.$pk_obj->getOrderTotal().'"/>
                <input type="hidden" name="orderid" value = "'.$pk_obj->getOrderParams("orderid").'"/>
                <input type="hidden" name="clientid" value = "'.$pk_obj->getOrderParams("clientid").'"/>
                <input type="hidden" name="client_email" value = "'.$pk_obj->getOrderParams("client_email").'"/>
                <input type="hidden" name="client_phone" value = "'.$pk_obj->getOrderParams("client_phone").'"/>
                <input type="hidden" name="service_name" value = "'.$pk_obj->getOrderParams("service_name").'"/>
                <input type="hidden" name="cart" value = \''.htmlentities($fiscal_cart_encoded,ENT_QUOTES).'\' />
                <input type="hidden" name="sign" value = "'.$sign.'"/> 
                <input type="submit" class="'.$this->getSubmitButtonCSSClass().'" value="Оплатить"/>
                </form>
                <script type="text/javascript">
                window.onload=function(){
                    setTimeout(fSubmit, 2000);
                }
                function fSubmit() {
                    document.forms["pay_form"].submit();
                }
                </script>
                ';
                
        }
        else { //order form
            $payment_parameters = array(
                "clientid"=>$pk_obj->getOrderParams("clientid"), 
                "orderid"=>$pk_obj->getOrderParams('orderid'), 
                "sum"=>$pk_obj->getOrderTotal(), 
                "phone"=>$pk_obj->getOrderParams("phone"), 
                "client_email"=>$pk_obj->getOrderParams("client_email"), 
                "cart"=>$fiscal_cart_encoded);
            $query = http_build_query($payment_parameters);
            $err_num = $err_text = NULL;
            if( function_exists( "curl_init" )) { //using curl
                $CR = curl_init();
                curl_setopt($CR, CURLOPT_URL, $pk_obj->getOrderParams("form_url"));
                curl_setopt($CR, CURLOPT_POST, 1);
                curl_setopt($CR, CURLOPT_FAILONERROR, true); 
                curl_setopt($CR, CURLOPT_POSTFIELDS, $query);
                curl_setopt($CR, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($CR, CURLOPT_SSL_VERIFYPEER, 0);
                $result = curl_exec( $CR );
                $error = curl_error( $CR );
                if( !empty( $error )) {
                    $form = "<br/><span class=message>"."INTERNAL ERROR:".$error."</span>";
                    return false;
                }
                else {
                    $form = $result;
                }
                curl_close($CR);
            }
            else { //using file_get_contents
                if (!ini_get('allow_url_fopen')) {
                    $form_html = "<br/><span class=message>"."INTERNAL ERROR: Option allow_url_fopen is not set in php.ini"."</span>";
                }
                else {
                    $form = file_get_contents($server, false, $context);
                }
            }
        }
        if ($form  == "") {
            $form = '<h3>Произошла ошибка при инциализации платежа</h3><p>$err_num: '.htmlspecialchars($err_text).'</p>';
        }

		//render form
        $view = wa()->getView();
		$view->assign('form_html', $form);
		return $view->fetch($this->path.'/templates/payment.html');
	}
	//
	//Инициируем callBack Функцию и подготавливаем переменные
	//
    protected function callbackInit($request)
    {
        waLog::dump($request);
        if (!empty($request['id']) && !empty($request['sum']) && !empty($request['clientid']) && !empty($request['orderid']) && !empty($request['key'])) {

			//parse orderid
            $orderid_data = $this->parseOrderId($request["orderid"]);
		 	$this->order_id = $orderid_data[1]; 
    		$this->app_id = $orderid_data[2]; 
    		$this->merchant_id = $orderid_data[3];
		 } else {
            self::log($this->id, array('error' => 'empty required field(s)'));
            throw new waPaymentException('Empty required field(s)');
        }
      return parent::callbackInit($request);  
    }

 	//
	//Выполняем проверку и выводим сообщения
	//Так же пишем в лог, или обновляем статус заказа в случае успеха
	//
 protected function callbackHandler($request)
    {
        // приводим данные о транзакции к универсальному виду
        $transaction_data = $this->formalizeData($request);
        self::log("logs1", $transaction_data);
		
		//ВЫПОЛНЯЕМ ПРОВЕРКИ
		$pk_create  = md5($request['id'].number_format($request['sum'], 2, '.', '').$request['clientid'].$request['orderid'].$this->pk_secret_key);
		
        $orderid_data = $this->parseOrderId($request["orderid"]);
        $request["orderid"] = $orderid_data[1];
		
		if(strcmp($pk_create, $request['key']) == 0) {
		#
		#
		#
		//Если все хорошо выполняем и выводим ответ для ПС
		$transaction_data['type'] = self::OPERATION_AUTH_CAPTURE;
		$transaction_data['state'] = self::STATE_CAPTURED;
		$callback_method = self::CALLBACK_PAYMENT;	
		$transaction_data = $this->saveTransaction($transaction_data, $request);
		$result = $this->execAppCallback($callback_method,$transaction_data);

        if (!empty($result['result'])) {
            self::log($this->id, array('result' => 'success'));
            echo $message = 'OK '.md5($request['id'].$this->pk_secret_key);
			exit();
        } else {
            $message = !empty($result['error']) ? $result['error'] : 'wa transaction error';
            self::log($this->id, array('error' => $message));
            //header("HTTP/1.0 403 Forbidden");
			echo $message;
			exit();
        }
		#
		#
		#
		} else {}
		//
			$message = 'Message digest incorrect';
            self::log($this->id, array('error' => $message));
            //header("HTTP/1.0 403 Forbidden");
			echo $message;
			exit();
    }

	#
	//
	//Подготавливаем массив для вставки в БД и определяем тип
	//
	#
    protected function formalizeData($request) {
	// формируем полный список полей, относящихся к транзакциям, которые обрабатываются платежной системой payKeeper
	$fields = array(
            'id',
            'sum',
            'orderid',
            'clientid',
            'key',

        ); 
        foreach ($fields as $f) {
            if (!isset($request[$f])) {
                $request[$f] = null;
            }
        }
	
	// выполняем базовую обработку данных
	$transaction_data = parent::formalizeData($request);
	//Тип транзацкии
	$transaction_data['type'] = self::OPERATION_AUTH_CAPTURE;
	// идентификатор транзакции, присвоенный платежной системой
	$transaction_data['native_id'] = $request['id'];
	// номер заказа
	//$pk_new_param = explode(':::',$request['orderid']);
    $orderid_data = $this->parseOrderId($request["orderid"]);
	$transaction_data['order_id'] = $orderid_data[1];
	// сумма заказа
	$transaction_data['amount'] = number_format($request['sum'], 2, '.', '');
	// идентификатор валюты заказа
	$transaction_data['currency_id'] = 'RUB';
	$transaction_data['result'] = 1;
	$transaction_data['view_data'] = 'Оплачено через PayKeeper - статус ответа: success';
	$transaction_data['state'] = self::STATE_CAPTURED;
	
	return $transaction_data;
	}
	
	protected function setSubmitButtonCSSClass($class_string)
    {
        $this->submit_button_class = $class_string;
    }

    protected function getSubmitButtonCSSClass()
    {
        return $this->submit_button_class;
    }

    protected function parseOrderId($orderid)
    {
        return explode("|", $orderid);
    }
}
